/*============================================================================
  File:     Scalar Functions.sql

  Summary:  This script shows the effects of a scalar function in a large
            join.

  Date:     March 2012

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE Credit
go

CREATE FUNCTION dbo.MemberName
	(@MemberNo	int)
RETURNS varchar(31)
AS
BEGIN
RETURN
	(SELECT m.FirstName + ' ' + m.LastName
	FROM dbo.Member AS m
	WHERE m.member_no = @MemberNo)
END
GO

DBCC FREEPROCCACHE
go

set statistics io on
set statistics time on
go

DECLARE @starttime  datetime2
SELECT @starttime = SYSDATETIME()

SELECT m.member_no
 , m.FirstName + ' ' + m.LastName
FROM dbo.member AS m
SELECT DurationQuery = DATEDIFF(ms, @starttime, SYSDATETIME()) 

SELECT @starttime = SYSDATETIME()
SELECT m.member_no
 , dbo.MemberName(m.member_no)
FROM dbo.member AS m
SELECT DurationFunction = DATEDIFF(ms, @starttime, SYSDATETIME()) 
go

-- But - it gets A LOT worse when you put this into more complicated statements
-- Forces SQL Server to execute row-by-row (ReBAR: Row by agonizing row)
DECLARE @starttime  datetime2
SELECT @starttime = SYSDATETIME()

SELECT c.member_no, m.FirstName + ' ' + m.LastName, sum(c.charge_amt)
FROM dbo.charge AS c
	JOIN dbo.member AS m ON c.member_no = m.member_no
GROUP BY c.member_no, m.FirstName + ' ' + m.LastName
SELECT DurationQuery = DATEDIFF(ms, @starttime, SYSDATETIME()) 

--Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
--Table 'charge'. Scan count 9, logical reads 9453, physical reads 43, read-ahead reads 6039, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
--Table 'member'. Scan count 9, logical reads 427, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

--SQL Server Execution Times:
--   CPU time = 1684 ms,  elapsed time = 583 ms.

SELECT @starttime = SYSDATETIME()
SELECT c.member_no, 
    dbo.MemberName(c.member_no), 
    sum(c.charge_amt)
FROM dbo.charge AS c
GROUP BY c.member_no, dbo.MemberName(c.member_no)
SELECT DurationFunction = DATEDIFF(ms, @starttime, SYSDATETIME()) 
go

--Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
--Table 'charge'. Scan count 1, logical reads 9335, physical reads 3057, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

-- SQL Server Execution Times:
--   CPU time = 989249 ms,  elapsed time = 1140247 ms. (roughly 19 mins)