/*============================================================================
  File:     Heuristics.sql

  Summary:  There are numerous cases where statistics cannot be used - so 
            then what? Heuristics (fixed calculations that are rules-based)
            are used...
  
  Date:     June 2011

  SQL Server Version: SQL Server 2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE Credit
go

SET STATISTICS PROFILE ON
go

-- Here - they yield unique because column unique
DECLARE @test   int
SELECT @test = 9999
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no = @test
go

-- Here - we have an index but no "sniffing"
-- use density_vector for "all density" (aka unknown)
-- simply put this is the average

-- with an index on LN
CREATE INDEX LNInd on member(LastName)
go
DECLARE @test   varchar(15)
SELECT @test = 'Tripp'
SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = @test
go

DBCC SHOW_STATISTICS ('member', 'LNind')
go

-- without index on LN AND w/o auto create stats
ALTER DATABASE [Credit] SET AUTO_CREATE_STATISTICS OFF
go
DROP INDEX member.LNInd
go
--DROP STATISTICS member.[_WA_Sys_00000002_0CBAE877]
--exec sp_helpstats member
go

DECLARE @test   varchar(15)
SELECT @test = 'Tripp'
SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = @test
go


-- Here - they yield 30% *no matter what*
DECLARE @test   int
SELECT @test = 9999
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no > @test
go

-- Again - they yield 30% 
DECLARE @test   int
SELECT @test = 1
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no < @test
go

-- Here - they yield 9% *no matter what*
DECLARE @lowerbound   int
DECLARE @upperbound   int
SELECT @lowerbound = 1
SELECT @upperbound = 10000

SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no BETWEEN @lowerbound AND @upperbound
go

-- What about a comparison between columns?
DROP TABLE member2
go

SELECT member_no as MNO1,
        member_no AS MNO2,
        lastname,
        firstname,
        middleinitial 
INTO member2
FROM member
go

SELECT m.* 
FROM dbo.member2 AS m
WHERE m.MNO1 = m.MNO2
-- Here it looks like the estimate is 10%...